@@ -1,16 +1,17 @@
-import os
-import io
-import json
-import platform
 import ipaddress
+import json
+import os
+import platform
+import sys
 import urllib.parse
+from hashlib import sha256
 from os import PathLike
 from pathlib import Path
-from hashlib import sha256
-
 from typing import (
   Any,
   Callable,
+  Dict,
+  List,
   Literal,
   Mapping,
   Optional,
@@ -21,17 +22,14 @@ from typing import (
   overload,
 )
 
-import sys
-
 from pydantic.json_schema import JsonSchemaValue
 
-
 from ollama._utils import convert_function_to_tool
 
 if sys.version_info < (3, 9):
-  from typing import Iterator, AsyncIterator
+  from typing import AsyncIterator, Iterator
 else:
-  from collections.abc import Iterator, AsyncIterator
+  from collections.abc import AsyncIterator, Iterator
 
 from importlib import metadata
 
@@ -45,13 +43,13 @@ import httpx
 from ollama._types import (
   ChatRequest,
   ChatResponse,
-  CreateRequest,
   CopyRequest,
+  CreateRequest,
   DeleteRequest,
-  EmbedRequest,
-  EmbedResponse,
   EmbeddingsRequest,
   EmbeddingsResponse,
+  EmbedRequest,
+  EmbedResponse,
   GenerateRequest,
   GenerateResponse,
   Image,
@@ -62,7 +60,6 @@ from ollama._types import (
   ProgressResponse,
   PullRequest,
   PushRequest,
-  RequestError,
   ResponseError,
   ShowRequest,
   ShowResponse,
@@ -70,7 +67,6 @@ from ollama._types import (
   Tool,
 )
 
-
 T = TypeVar('T')
 
 
@@ -110,17 +106,22 @@ class BaseClient:
     )
 
 
+CONNECTION_ERROR_MESSAGE = 'Failed to connect to Ollama. Please check that Ollama is downloaded, running and accessible. https://ollama.com/download'
+
+
 class Client(BaseClient):
   def __init__(self, host: Optional[str] = None, **kwargs) -> None:
     super().__init__(httpx.Client, host, **kwargs)
 
   def _request_raw(self, *args, **kwargs):
-    r = self._client.request(*args, **kwargs)
     try:
+      r = self._client.request(*args, **kwargs)
       r.raise_for_status()
+      return r
     except httpx.HTTPStatusError as e:
       raise ResponseError(e.response.text, e.response.status_code) from None
-    return r
+    except httpx.ConnectError:
+      raise ConnectionError(CONNECTION_ERROR_MESSAGE) from None
 
   @overload
   def _request(
@@ -476,10 +477,16 @@ class Client(BaseClient):
   def create(
     self,
     model: str,
-    path: Optional[Union[str, PathLike]] = None,
-    modelfile: Optional[str] = None,
-    *,
     quantize: Optional[str] = None,
+    from_: Optional[str] = None,
+    files: Optional[Dict[str, str]] = None,
+    adapters: Optional[Dict[str, str]] = None,
+    template: Optional[str] = None,
+    license: Optional[Union[str, List[str]]] = None,
+    system: Optional[str] = None,
+    parameters: Optional[Union[Mapping[str, Any], Options]] = None,
+    messages: Optional[Sequence[Union[Mapping[str, Any], Message]]] = None,
+    *,
     stream: Literal[False] = False,
   ) -> ProgressResponse: ...
 
@@ -487,20 +494,32 @@ class Client(BaseClient):
   def create(
     self,
     model: str,
-    path: Optional[Union[str, PathLike]] = None,
-    modelfile: Optional[str] = None,
-    *,
     quantize: Optional[str] = None,
+    from_: Optional[str] = None,
+    files: Optional[Dict[str, str]] = None,
+    adapters: Optional[Dict[str, str]] = None,
+    template: Optional[str] = None,
+    license: Optional[Union[str, List[str]]] = None,
+    system: Optional[str] = None,
+    parameters: Optional[Union[Mapping[str, Any], Options]] = None,
+    messages: Optional[Sequence[Union[Mapping[str, Any], Message]]] = None,
+    *,
     stream: Literal[True] = True,
   ) -> Iterator[ProgressResponse]: ...
 
   def create(
     self,
     model: str,
-    path: Optional[Union[str, PathLike]] = None,
-    modelfile: Optional[str] = None,
-    *,
     quantize: Optional[str] = None,
+    from_: Optional[str] = None,
+    files: Optional[Dict[str, str]] = None,
+    adapters: Optional[Dict[str, str]] = None,
+    template: Optional[str] = None,
+    license: Optional[Union[str, List[str]]] = None,
+    system: Optional[str] = None,
+    parameters: Optional[Union[Mapping[str, Any], Options]] = None,
+    messages: Optional[Sequence[Union[Mapping[str, Any], Message]]] = None,
+    *,
     stream: bool = False,
   ) -> Union[ProgressResponse, Iterator[ProgressResponse]]:
     """
@@ -508,45 +527,27 @@ class Client(BaseClient):
 
     Returns `ProgressResponse` if `stream` is `False`, otherwise returns a `ProgressResponse` generator.
     """
-    if (realpath := _as_path(path)) and realpath.exists():
-      modelfile = self._parse_modelfile(realpath.read_text(), base=realpath.parent)
-    elif modelfile:
-      modelfile = self._parse_modelfile(modelfile)
-    else:
-      raise RequestError('must provide either path or modelfile')
-
     return self._request(
       ProgressResponse,
       'POST',
       '/api/create',
       json=CreateRequest(
         model=model,
-        modelfile=modelfile,
         stream=stream,
         quantize=quantize,
+        from_=from_,
+        files=files,
+        adapters=adapters,
+        license=license,
+        template=template,
+        system=system,
+        parameters=parameters,
+        messages=messages,
       ).model_dump(exclude_none=True),
       stream=stream,
     )
 
-  def _parse_modelfile(self, modelfile: str, base: Optional[Path] = None) -> str:
-    base = Path.cwd() if base is None else base
-
-    out = io.StringIO()
-    for line in io.StringIO(modelfile):
-      command, _, args = line.partition(' ')
-      if command.upper() not in ['FROM', 'ADAPTER']:
-        print(line, end='', file=out)
-        continue
-
-      path = Path(args.strip()).expanduser()
-      path = path if path.is_absolute() else base / path
-      if path.exists():
-        args = f'@{self._create_blob(path)}\n'
-      print(command, args, end='', file=out)
-
-    return out.getvalue()
-
-  def _create_blob(self, path: Union[str, Path]) -> str:
+  def create_blob(self, path: Union[str, Path]) -> str:
     sha256sum = sha256()
     with open(path, 'rb') as r:
       while True:
@@ -558,7 +559,7 @@ class Client(BaseClient):
     digest = f'sha256:{sha256sum.hexdigest()}'
 
     with open(path, 'rb') as r:
-      self._request_raw('POST', f'/api/blobs/sha256:{digest}', content=r)
+      self._request_raw('POST', f'/api/blobs/{digest}', content=r)
 
     return digest
 
@@ -617,12 +618,14 @@ class AsyncClient(BaseClient):
     super().__init__(httpx.AsyncClient, host, **kwargs)
 
   async def _request_raw(self, *args, **kwargs):
-    r = await self._client.request(*args, **kwargs)
     try:
+      r = await self._client.request(*args, **kwargs)
       r.raise_for_status()
+      return r
     except httpx.HTTPStatusError as e:
       raise ResponseError(e.response.text, e.response.status_code) from None
-    return r
+    except httpx.ConnectError:
+      raise ConnectionError(CONNECTION_ERROR_MESSAGE) from None
 
   @overload
   async def _request(
@@ -978,31 +981,49 @@ class AsyncClient(BaseClient):
   async def create(
     self,
     model: str,
-    path: Optional[Union[str, PathLike]] = None,
-    modelfile: Optional[str] = None,
-    *,
     quantize: Optional[str] = None,
-    stream: Literal[False] = False,
+    from_: Optional[str] = None,
+    files: Optional[Dict[str, str]] = None,
+    adapters: Optional[Dict[str, str]] = None,
+    template: Optional[str] = None,
+    license: Optional[Union[str, List[str]]] = None,
+    system: Optional[str] = None,
+    parameters: Optional[Union[Mapping[str, Any], Options]] = None,
+    messages: Optional[Sequence[Union[Mapping[str, Any], Message]]] = None,
+    *,
+    stream: Literal[True] = True,
   ) -> ProgressResponse: ...
 
   @overload
   async def create(
     self,
     model: str,
-    path: Optional[Union[str, PathLike]] = None,
-    modelfile: Optional[str] = None,
-    *,
     quantize: Optional[str] = None,
+    from_: Optional[str] = None,
+    files: Optional[Dict[str, str]] = None,
+    adapters: Optional[Dict[str, str]] = None,
+    template: Optional[str] = None,
+    license: Optional[Union[str, List[str]]] = None,
+    system: Optional[str] = None,
+    parameters: Optional[Union[Mapping[str, Any], Options]] = None,
+    messages: Optional[Sequence[Union[Mapping[str, Any], Message]]] = None,
+    *,
     stream: Literal[True] = True,
   ) -> AsyncIterator[ProgressResponse]: ...
 
   async def create(
     self,
     model: str,
-    path: Optional[Union[str, PathLike]] = None,
-    modelfile: Optional[str] = None,
-    *,
     quantize: Optional[str] = None,
+    from_: Optional[str] = None,
+    files: Optional[Dict[str, str]] = None,
+    adapters: Optional[Dict[str, str]] = None,
+    template: Optional[str] = None,
+    license: Optional[Union[str, List[str]]] = None,
+    system: Optional[str] = None,
+    parameters: Optional[Union[Mapping[str, Any], Options]] = None,
+    messages: Optional[Sequence[Union[Mapping[str, Any], Message]]] = None,
+    *,
     stream: bool = False,
   ) -> Union[ProgressResponse, AsyncIterator[ProgressResponse]]:
     """
@@ -1010,12 +1031,6 @@ class AsyncClient(BaseClient):
 
     Returns `ProgressResponse` if `stream` is `False`, otherwise returns a `ProgressResponse` generator.
     """
-    if (realpath := _as_path(path)) and realpath.exists():
-      modelfile = await self._parse_modelfile(realpath.read_text(), base=realpath.parent)
-    elif modelfile:
-      modelfile = await self._parse_modelfile(modelfile)
-    else:
-      raise RequestError('must provide either path or modelfile')
 
     return await self._request(
       ProgressResponse,
@@ -1023,32 +1038,21 @@ class AsyncClient(BaseClient):
       '/api/create',
       json=CreateRequest(
         model=model,
-        modelfile=modelfile,
         stream=stream,
         quantize=quantize,
+        from_=from_,
+        files=files,
+        adapters=adapters,
+        license=license,
+        template=template,
+        system=system,
+        parameters=parameters,
+        messages=messages,
       ).model_dump(exclude_none=True),
       stream=stream,
     )
 
-  async def _parse_modelfile(self, modelfile: str, base: Optional[Path] = None) -> str:
-    base = Path.cwd() if base is None else base
-
-    out = io.StringIO()
-    for line in io.StringIO(modelfile):
-      command, _, args = line.partition(' ')
-      if command.upper() not in ['FROM', 'ADAPTER']:
-        print(line, end='', file=out)
-        continue
-
-      path = Path(args.strip()).expanduser()
-      path = path if path.is_absolute() else base / path
-      if path.exists():
-        args = f'@{await self._create_blob(path)}\n'
-      print(command, args, end='', file=out)
-
-    return out.getvalue()
-
-  async def _create_blob(self, path: Union[str, Path]) -> str:
+  async def create_blob(self, path: Union[str, Path]) -> str:
     sha256sum = sha256()
     with open(path, 'rb') as r:
       while True:
