@@ -1,14 +1,16 @@
 import base64
-import os
 import json
-from pydantic import ValidationError, BaseModel
-import pytest
+import os
+import re
 import tempfile
 from pathlib import Path
+
+import pytest
+from pydantic import BaseModel, ValidationError
 from pytest_httpserver import HTTPServer, URIPattern
 from werkzeug.wrappers import Request, Response
 
-from ollama._client import Client, AsyncClient, _copy_tools
+from ollama._client import CONNECTION_ERROR_MESSAGE, AsyncClient, Client, _copy_tools
 
 PNG_BASE64 = 'iVBORw0KGgoAAAANSUhEUgAAAAEAAAABCAIAAACQd1PeAAAADElEQVR4nGNgYGAAAAAEAAH2FzhVAAAAAElFTkSuQmCC'
 PNG_BYTES = base64.b64decode(PNG_BASE64)
@@ -91,7 +93,7 @@ def test_client_chat_stream(httpserver: HTTPServer):
 @pytest.mark.parametrize('message_format', ('dict', 'pydantic_model'))
 @pytest.mark.parametrize('file_style', ('path', 'bytes'))
 def test_client_chat_images(httpserver: HTTPServer, message_format: str, file_style: str, tmp_path):
-  from ollama._types import Message, Image
+  from ollama._types import Image, Message
 
   httpserver.expect_ordered_request(
     '/api/chat',
@@ -536,52 +538,6 @@ def test_client_push_stream(httpserver: HTTPServer):
     assert part['status'] == next(it)
 
 
-def test_client_create_path(httpserver: HTTPServer):
-  httpserver.expect_ordered_request(PrefixPattern('/api/blobs/'), method='POST').respond_with_response(Response(status=200))
-  httpserver.expect_ordered_request(
-    '/api/create',
-    method='POST',
-    json={
-      'model': 'dummy',
-      'modelfile': 'FROM @sha256:e3b0c44298fc1c149afbf4c8996fb92427ae41e4649b934ca495991b7852b855\n',
-      'stream': False,
-    },
-  ).respond_with_json({'status': 'success'})
-
-  client = Client(httpserver.url_for('/'))
-
-  with tempfile.NamedTemporaryFile() as modelfile:
-    with tempfile.NamedTemporaryFile() as blob:
-      modelfile.write(f'FROM {blob.name}'.encode('utf-8'))
-      modelfile.flush()
-
-      response = client.create('dummy', path=modelfile.name)
-      assert response['status'] == 'success'
-
-
-def test_client_create_path_relative(httpserver: HTTPServer):
-  httpserver.expect_ordered_request(PrefixPattern('/api/blobs/'), method='POST').respond_with_response(Response(status=200))
-  httpserver.expect_ordered_request(
-    '/api/create',
-    method='POST',
-    json={
-      'model': 'dummy',
-      'modelfile': 'FROM @sha256:e3b0c44298fc1c149afbf4c8996fb92427ae41e4649b934ca495991b7852b855\n',
-      'stream': False,
-    },
-  ).respond_with_json({'status': 'success'})
-
-  client = Client(httpserver.url_for('/'))
-
-  with tempfile.NamedTemporaryFile() as modelfile:
-    with tempfile.NamedTemporaryFile(dir=Path(modelfile.name).parent) as blob:
-      modelfile.write(f'FROM {Path(blob.name).name}'.encode('utf-8'))
-      modelfile.flush()
-
-      response = client.create('dummy', path=modelfile.name)
-      assert response['status'] == 'success'
-
-
 @pytest.fixture
 def userhomedir():
   with tempfile.TemporaryDirectory() as temp:
@@ -591,92 +547,56 @@ def userhomedir():
     os.environ['HOME'] = home
 
 
-def test_client_create_path_user_home(httpserver: HTTPServer, userhomedir):
-  httpserver.expect_ordered_request(PrefixPattern('/api/blobs/'), method='POST').respond_with_response(Response(status=200))
+def test_client_create_with_blob(httpserver: HTTPServer):
   httpserver.expect_ordered_request(
     '/api/create',
     method='POST',
     json={
       'model': 'dummy',
-      'modelfile': 'FROM @sha256:e3b0c44298fc1c149afbf4c8996fb92427ae41e4649b934ca495991b7852b855\n',
+      'files': {'test.gguf': 'sha256:e3b0c44298fc1c149afbf4c8996fb92427ae41e4649b934ca495991b7852b855'},
       'stream': False,
     },
   ).respond_with_json({'status': 'success'})
 
   client = Client(httpserver.url_for('/'))
 
-  with tempfile.NamedTemporaryFile() as modelfile:
-    with tempfile.NamedTemporaryFile(dir=userhomedir) as blob:
-      modelfile.write(f'FROM ~/{Path(blob.name).name}'.encode('utf-8'))
-      modelfile.flush()
-
-      response = client.create('dummy', path=modelfile.name)
-      assert response['status'] == 'success'
-
-
-def test_client_create_modelfile(httpserver: HTTPServer):
-  httpserver.expect_ordered_request(PrefixPattern('/api/blobs/'), method='POST').respond_with_response(Response(status=200))
-  httpserver.expect_ordered_request(
-    '/api/create',
-    method='POST',
-    json={
-      'model': 'dummy',
-      'modelfile': 'FROM @sha256:e3b0c44298fc1c149afbf4c8996fb92427ae41e4649b934ca495991b7852b855\n',
-      'stream': False,
-    },
-  ).respond_with_json({'status': 'success'})
-
-  client = Client(httpserver.url_for('/'))
-
-  with tempfile.NamedTemporaryFile() as blob:
-    response = client.create('dummy', modelfile=f'FROM {blob.name}')
+  with tempfile.NamedTemporaryFile():
+    response = client.create('dummy', files={'test.gguf': 'sha256:e3b0c44298fc1c149afbf4c8996fb92427ae41e4649b934ca495991b7852b855'})
     assert response['status'] == 'success'
 
 
-def test_client_create_modelfile_roundtrip(httpserver: HTTPServer):
-  httpserver.expect_ordered_request(PrefixPattern('/api/blobs/'), method='POST').respond_with_response(Response(status=200))
+def test_client_create_with_parameters_roundtrip(httpserver: HTTPServer):
   httpserver.expect_ordered_request(
     '/api/create',
     method='POST',
     json={
       'model': 'dummy',
-      'modelfile': '''FROM @sha256:e3b0c44298fc1c149afbf4c8996fb92427ae41e4649b934ca495991b7852b855
-TEMPLATE """[INST] <<SYS>>{{.System}}<</SYS>>
-{{.Prompt}} [/INST]"""
-SYSTEM """
-Use
-multiline
-strings.
-"""
-PARAMETER stop [INST]
-PARAMETER stop [/INST]
-PARAMETER stop <<SYS>>
-PARAMETER stop <</SYS>>''',
+      'quantize': 'q4_k_m',
+      'from': 'mymodel',
+      'adapters': {'someadapter.gguf': 'sha256:e3b0c44298fc1c149afbf4c8996fb92427ae41e4649b934ca495991b7852b855'},
+      'template': '[INST] <<SYS>>{{.System}}<</SYS>>\n{{.Prompt}} [/INST]',
+      'license': 'this is my license',
+      'system': '\nUse\nmultiline\nstrings.\n',
+      'parameters': {'stop': ['[INST]', '[/INST]', '<<SYS>>', '<</SYS>>'], 'pi': 3.14159},
+      'messages': [{'role': 'user', 'content': 'Hello there!'}, {'role': 'assistant', 'content': 'Hello there yourself!'}],
       'stream': False,
     },
   ).respond_with_json({'status': 'success'})
 
   client = Client(httpserver.url_for('/'))
 
-  with tempfile.NamedTemporaryFile() as blob:
+  with tempfile.NamedTemporaryFile():
     response = client.create(
       'dummy',
-      modelfile='\n'.join(
-        [
-          f'FROM {blob.name}',
-          'TEMPLATE """[INST] <<SYS>>{{.System}}<</SYS>>',
-          '{{.Prompt}} [/INST]"""',
-          'SYSTEM """',
-          'Use',
-          'multiline',
-          'strings.',
-          '"""',
-          'PARAMETER stop [INST]',
-          'PARAMETER stop [/INST]',
-          'PARAMETER stop <<SYS>>',
-          'PARAMETER stop <</SYS>>',
-        ]
-      ),
+      quantize='q4_k_m',
+      from_='mymodel',
+      adapters={'someadapter.gguf': 'sha256:e3b0c44298fc1c149afbf4c8996fb92427ae41e4649b934ca495991b7852b855'},
+      template='[INST] <<SYS>>{{.System}}<</SYS>>\n{{.Prompt}} [/INST]',
+      license='this is my license',
+      system='\nUse\nmultiline\nstrings.\n',
+      parameters={'stop': ['[INST]', '[/INST]', '<<SYS>>', '<</SYS>>'], 'pi': 3.14159},
+      messages=[{'role': 'user', 'content': 'Hello there!'}, {'role': 'assistant', 'content': 'Hello there yourself!'}],
+      stream=False,
     )
     assert response['status'] == 'success'
 
@@ -687,24 +607,24 @@ def test_client_create_from_library(httpserver: HTTPServer):
     method='POST',
     json={
       'model': 'dummy',
-      'modelfile': 'FROM llama2',
+      'from': 'llama2',
       'stream': False,
     },
   ).respond_with_json({'status': 'success'})
 
   client = Client(httpserver.url_for('/'))
 
-  response = client.create('dummy', modelfile='FROM llama2')
+  response = client.create('dummy', from_='llama2')
   assert response['status'] == 'success'
 
 
 def test_client_create_blob(httpserver: HTTPServer):
-  httpserver.expect_ordered_request(PrefixPattern('/api/blobs/'), method='POST').respond_with_response(Response(status=201))
+  httpserver.expect_ordered_request(re.compile('^/api/blobs/sha256[:-][0-9a-fA-F]{64}$'), method='POST').respond_with_response(Response(status=201))
 
   client = Client(httpserver.url_for('/'))
 
   with tempfile.NamedTemporaryFile() as blob:
-    response = client._create_blob(blob.name)
+    response = client.create_blob(blob.name)
     assert response == 'sha256:e3b0c44298fc1c149afbf4c8996fb92427ae41e4649b934ca495991b7852b855'
 
 
@@ -714,7 +634,7 @@ def test_client_create_blob_exists(httpserver: HTTPServer):
   client = Client(httpserver.url_for('/'))
 
   with tempfile.NamedTemporaryFile() as blob:
-    response = client._create_blob(blob.name)
+    response = client.create_blob(blob.name)
     assert response == 'sha256:e3b0c44298fc1c149afbf4c8996fb92427ae41e4649b934ca495991b7852b855'
 
 
@@ -1015,142 +935,57 @@ async def test_async_client_push_stream(httpserver: HTTPServer):
 
 
 @pytest.mark.asyncio
-async def test_async_client_create_path(httpserver: HTTPServer):
-  httpserver.expect_ordered_request(PrefixPattern('/api/blobs/'), method='POST').respond_with_response(Response(status=200))
+async def test_async_client_create_with_blob(httpserver: HTTPServer):
   httpserver.expect_ordered_request(
     '/api/create',
     method='POST',
     json={
       'model': 'dummy',
-      'modelfile': 'FROM @sha256:e3b0c44298fc1c149afbf4c8996fb92427ae41e4649b934ca495991b7852b855\n',
+      'files': {'test.gguf': 'sha256:e3b0c44298fc1c149afbf4c8996fb92427ae41e4649b934ca495991b7852b855'},
       'stream': False,
     },
   ).respond_with_json({'status': 'success'})
 
   client = AsyncClient(httpserver.url_for('/'))
 
-  with tempfile.NamedTemporaryFile() as modelfile:
-    with tempfile.NamedTemporaryFile() as blob:
-      modelfile.write(f'FROM {blob.name}'.encode('utf-8'))
-      modelfile.flush()
-
-      response = await client.create('dummy', path=modelfile.name)
-      assert response['status'] == 'success'
-
-
-@pytest.mark.asyncio
-async def test_async_client_create_path_relative(httpserver: HTTPServer):
-  httpserver.expect_ordered_request(PrefixPattern('/api/blobs/'), method='POST').respond_with_response(Response(status=200))
-  httpserver.expect_ordered_request(
-    '/api/create',
-    method='POST',
-    json={
-      'model': 'dummy',
-      'modelfile': 'FROM @sha256:e3b0c44298fc1c149afbf4c8996fb92427ae41e4649b934ca495991b7852b855\n',
-      'stream': False,
-    },
-  ).respond_with_json({'status': 'success'})
-
-  client = AsyncClient(httpserver.url_for('/'))
-
-  with tempfile.NamedTemporaryFile() as modelfile:
-    with tempfile.NamedTemporaryFile(dir=Path(modelfile.name).parent) as blob:
-      modelfile.write(f'FROM {Path(blob.name).name}'.encode('utf-8'))
-      modelfile.flush()
-
-      response = await client.create('dummy', path=modelfile.name)
-      assert response['status'] == 'success'
-
-
-@pytest.mark.asyncio
-async def test_async_client_create_path_user_home(httpserver: HTTPServer, userhomedir):
-  httpserver.expect_ordered_request(PrefixPattern('/api/blobs/'), method='POST').respond_with_response(Response(status=200))
-  httpserver.expect_ordered_request(
-    '/api/create',
-    method='POST',
-    json={
-      'model': 'dummy',
-      'modelfile': 'FROM @sha256:e3b0c44298fc1c149afbf4c8996fb92427ae41e4649b934ca495991b7852b855\n',
-      'stream': False,
-    },
-  ).respond_with_json({'status': 'success'})
-
-  client = AsyncClient(httpserver.url_for('/'))
-
-  with tempfile.NamedTemporaryFile() as modelfile:
-    with tempfile.NamedTemporaryFile(dir=userhomedir) as blob:
-      modelfile.write(f'FROM ~/{Path(blob.name).name}'.encode('utf-8'))
-      modelfile.flush()
-
-      response = await client.create('dummy', path=modelfile.name)
-      assert response['status'] == 'success'
-
-
-@pytest.mark.asyncio
-async def test_async_client_create_modelfile(httpserver: HTTPServer):
-  httpserver.expect_ordered_request(PrefixPattern('/api/blobs/'), method='POST').respond_with_response(Response(status=200))
-  httpserver.expect_ordered_request(
-    '/api/create',
-    method='POST',
-    json={
-      'model': 'dummy',
-      'modelfile': 'FROM @sha256:e3b0c44298fc1c149afbf4c8996fb92427ae41e4649b934ca495991b7852b855\n',
-      'stream': False,
-    },
-  ).respond_with_json({'status': 'success'})
-
-  client = AsyncClient(httpserver.url_for('/'))
-
-  with tempfile.NamedTemporaryFile() as blob:
-    response = await client.create('dummy', modelfile=f'FROM {blob.name}')
+  with tempfile.NamedTemporaryFile():
+    response = await client.create('dummy', files={'test.gguf': 'sha256:e3b0c44298fc1c149afbf4c8996fb92427ae41e4649b934ca495991b7852b855'})
     assert response['status'] == 'success'
 
 
 @pytest.mark.asyncio
-async def test_async_client_create_modelfile_roundtrip(httpserver: HTTPServer):
-  httpserver.expect_ordered_request(PrefixPattern('/api/blobs/'), method='POST').respond_with_response(Response(status=200))
+async def test_async_client_create_with_parameters_roundtrip(httpserver: HTTPServer):
   httpserver.expect_ordered_request(
     '/api/create',
     method='POST',
     json={
       'model': 'dummy',
-      'modelfile': '''FROM @sha256:e3b0c44298fc1c149afbf4c8996fb92427ae41e4649b934ca495991b7852b855
-TEMPLATE """[INST] <<SYS>>{{.System}}<</SYS>>
-{{.Prompt}} [/INST]"""
-SYSTEM """
-Use
-multiline
-strings.
-"""
-PARAMETER stop [INST]
-PARAMETER stop [/INST]
-PARAMETER stop <<SYS>>
-PARAMETER stop <</SYS>>''',
+      'quantize': 'q4_k_m',
+      'from': 'mymodel',
+      'adapters': {'someadapter.gguf': 'sha256:e3b0c44298fc1c149afbf4c8996fb92427ae41e4649b934ca495991b7852b855'},
+      'template': '[INST] <<SYS>>{{.System}}<</SYS>>\n{{.Prompt}} [/INST]',
+      'license': 'this is my license',
+      'system': '\nUse\nmultiline\nstrings.\n',
+      'parameters': {'stop': ['[INST]', '[/INST]', '<<SYS>>', '<</SYS>>'], 'pi': 3.14159},
+      'messages': [{'role': 'user', 'content': 'Hello there!'}, {'role': 'assistant', 'content': 'Hello there yourself!'}],
       'stream': False,
     },
   ).respond_with_json({'status': 'success'})
 
   client = AsyncClient(httpserver.url_for('/'))
 
-  with tempfile.NamedTemporaryFile() as blob:
+  with tempfile.NamedTemporaryFile():
     response = await client.create(
       'dummy',
-      modelfile='\n'.join(
-        [
-          f'FROM {blob.name}',
-          'TEMPLATE """[INST] <<SYS>>{{.System}}<</SYS>>',
-          '{{.Prompt}} [/INST]"""',
-          'SYSTEM """',
-          'Use',
-          'multiline',
-          'strings.',
-          '"""',
-          'PARAMETER stop [INST]',
-          'PARAMETER stop [/INST]',
-          'PARAMETER stop <<SYS>>',
-          'PARAMETER stop <</SYS>>',
-        ]
-      ),
+      quantize='q4_k_m',
+      from_='mymodel',
+      adapters={'someadapter.gguf': 'sha256:e3b0c44298fc1c149afbf4c8996fb92427ae41e4649b934ca495991b7852b855'},
+      template='[INST] <<SYS>>{{.System}}<</SYS>>\n{{.Prompt}} [/INST]',
+      license='this is my license',
+      system='\nUse\nmultiline\nstrings.\n',
+      parameters={'stop': ['[INST]', '[/INST]', '<<SYS>>', '<</SYS>>'], 'pi': 3.14159},
+      messages=[{'role': 'user', 'content': 'Hello there!'}, {'role': 'assistant', 'content': 'Hello there yourself!'}],
+      stream=False,
     )
     assert response['status'] == 'success'
 
@@ -1162,25 +997,25 @@ async def test_async_client_create_from_library(httpserver: HTTPServer):
     method='POST',
     json={
       'model': 'dummy',
-      'modelfile': 'FROM llama2',
+      'from': 'llama2',
       'stream': False,
     },
   ).respond_with_json({'status': 'success'})
 
   client = AsyncClient(httpserver.url_for('/'))
 
-  response = await client.create('dummy', modelfile='FROM llama2')
+  response = await client.create('dummy', from_='llama2')
   assert response['status'] == 'success'
 
 
 @pytest.mark.asyncio
 async def test_async_client_create_blob(httpserver: HTTPServer):
-  httpserver.expect_ordered_request(PrefixPattern('/api/blobs/'), method='POST').respond_with_response(Response(status=201))
+  httpserver.expect_ordered_request(re.compile('^/api/blobs/sha256[:-][0-9a-fA-F]{64}$'), method='POST').respond_with_response(Response(status=201))
 
   client = AsyncClient(httpserver.url_for('/'))
 
   with tempfile.NamedTemporaryFile() as blob:
-    response = await client._create_blob(blob.name)
+    response = await client.create_blob(blob.name)
     assert response == 'sha256:e3b0c44298fc1c149afbf4c8996fb92427ae41e4649b934ca495991b7852b855'
 
 
@@ -1191,7 +1026,7 @@ async def test_async_client_create_blob_exists(httpserver: HTTPServer):
   client = AsyncClient(httpserver.url_for('/'))
 
   with tempfile.NamedTemporaryFile() as blob:
-    response = await client._create_blob(blob.name)
+    response = await client.create_blob(blob.name)
     assert response == 'sha256:e3b0c44298fc1c149afbf4c8996fb92427ae41e4649b934ca495991b7852b855'
 
 
@@ -1278,3 +1113,30 @@ def test_tool_validation():
   with pytest.raises(ValidationError):
     invalid_tool = {'type': 'invalid_type', 'function': {'name': 'test'}}
     list(_copy_tools([invalid_tool]))
+
+
+def test_client_connection_error():
+  client = Client('http://localhost:1234')
+
+  with pytest.raises(ConnectionError, match=CONNECTION_ERROR_MESSAGE):
+    client.chat('model', messages=[{'role': 'user', 'content': 'prompt'}])
+  with pytest.raises(ConnectionError, match=CONNECTION_ERROR_MESSAGE):
+    client.chat('model', messages=[{'role': 'user', 'content': 'prompt'}])
+  with pytest.raises(ConnectionError, match=CONNECTION_ERROR_MESSAGE):
+    client.generate('model', 'prompt')
+  with pytest.raises(ConnectionError, match=CONNECTION_ERROR_MESSAGE):
+    client.show('model')
+
+
+@pytest.mark.asyncio
+async def test_async_client_connection_error():
+  client = AsyncClient('http://localhost:1234')
+  with pytest.raises(ConnectionError) as exc_info:
+    await client.chat('model', messages=[{'role': 'user', 'content': 'prompt'}])
+  assert str(exc_info.value) == 'Failed to connect to Ollama. Please check that Ollama is downloaded, running and accessible. https://ollama.com/download'
+  with pytest.raises(ConnectionError) as exc_info:
+    await client.generate('model', 'prompt')
+  assert str(exc_info.value) == 'Failed to connect to Ollama. Please check that Ollama is downloaded, running and accessible. https://ollama.com/download'
+  with pytest.raises(ConnectionError) as exc_info:
+    await client.show('model')
+  assert str(exc_info.value) == 'Failed to connect to Ollama. Please check that Ollama is downloaded, running and accessible. https://ollama.com/download'
